@@ -1,5 +1,7 @@
 from novelai_api import NovelAI_API
 from novelai_api.utils import encrypt_user_data
+from novelai_api.keystore import Keystore
+from novelai_api.idstore import Idstore
 
 from nacl.secret import SecretBox
 from nacl.utils import random
@@ -7,7 +9,7 @@ from nacl.utils import random
 from typing import Dict, List, NoReturn, Any, Optional, Union
 from uuid import uuid4
 from time import time
-
+from json import loads
 
 DEFAULT_MODEL = "6B-v3"
 DEFAULT_PARAMS = {
@@ -16,7 +18,7 @@ DEFAULT_PARAMS = {
 	"max_length": 30
 }
 
-def get_time() -> int:
+def _get_time() -> int:
 	"""
 	Get the current time, as formatted for createdAt and lastUpdatedAt
 
@@ -25,7 +27,7 @@ def get_time() -> int:
 
 	return int(time() * 1000)
 
-def get_short_time() -> int:
+def _get_short_time() -> int:
 	"""
 	Because some lastUpdatedAt only are precise to the second
 
@@ -34,92 +36,13 @@ def get_short_time() -> int:
 
 	return int(time())
 
-# FIXME: store the constants, like versions, in a global constants file
-def create_new_stories_data(meta: str, current_time: int, current_time_short: int) -> Dict[str, Any]:
-	return {
-		"id": None,
-		"type": "stories",
-		"meta": meta,
-		"data": {
-			"storyMetadatVersion": 1,
-			#what to do with id, remoteId, remoteStoryId ?
-			"title": "New story",
-			"description": "",
-			"textPreview": "",
-			"favorite": False,
-			"tags": [],
-			"createdAt": current_time,
-			"lastUpdatedAt": current_time,
-			"isModified": False
-		},
-		"lastUpdatedAt": current_time_short,
-		"changedIndex": 0,
-		"nonce": random(SecretBox.NONCE_SIZE),
-		"decrypted": True,
-		"compressed": False
-	}
+def _set_nested_item(item: Dict[str, Any], val: Any, path: str):
+	path = path.split('.')
 
-def create_new_storycontent_data(meta: str, current_time_short: int) -> Dict[str, Any]:
-	return {
-		"id": None,
-		"storycontent": "storycontent",
-		"meta": meta,
-		"data": {
-			"storyContentVersion": 3,
-			"settings": {
-				"parameters": {},			# FIXME: default preset ?
-				"preset": "",
-				"trimResponse": True,
-				"banBrackets": True,
-				"prefix": ""
-			},
-			# FIXME: refactor this into a function
-			"story": {
-				"version": 2,
-				"step": 0,
-				"datablocks": [ {
-					"nextBlock": [],
-					"prevBlock": -1,
-					"origin": "root",
-					"startIndex": 0,
-					"endIndex": 0,
-					"dataFragment": { "data": "", "origin": "root" },
-					"fragmentIndex": -1,
-					"removedFragments": [],
-					"chain": False
-				} ],
-				"currentBlock": 0,
-				"fragments": [ { "data": "", "origin": "root" } ]
-			},
-			"context": [
-				{
-					"text": "",
-					"contextConfig": {},		# FIXME: fill me
-				},
-				{
-					"text": "",
-					"contextConfig": {},		# FIXME: fill me
-				}
-			],
-			"lorebook": {
-				"lorebookVersion": 3,
-				"entries": [],
-				"settings": { "orderByKeyLocations": False },
-				"categories": []
-			},
-			"storyContextConfig": {},			# FIXME: fill me
-			"ephemeralContext": [],
-			"contextDefaults": {
-				"ephemeralDefaults": [ {} ],	# FIXME: fill me
-				"loreDefaults": [ {} ],			# FIXME: fill me
-			},
-			"settingsDirty": False
-		},
-		"lastUpdatedAt": current_time_short,
-		"changeIndex": 0,
-		"decrypted": True,
-		"compressed": True
-	}
+	for key in path[:-1]:
+		item = item[key]
+
+	item[path[-1]] = val
 
 class NovelAI_StoryProxy:
 	_parent: "NovelAI_Story"
@@ -128,6 +51,7 @@ class NovelAI_StoryProxy:
 	_key: bytes
 	_story: Dict[str, Any]
 	_storycontent: Dict[str, Any]
+	_tree: List[int]
 	model: str
 
 	def __init__(self, parent: "NovelAI_Story", key: bytes, story: Dict[str, Any], storycontent: Dict[str, Any], model: Optional[str] = None):
@@ -137,12 +61,11 @@ class NovelAI_StoryProxy:
 		self._key = key
 		self._story = story
 		self._storycontent = storycontent
+		self._tree = []
 
 		self._model = DEFAULT_MODEL if model is None else model
 
-	async def generate(self, input: Union[str, List[int]]) -> "NovelAI_StoryProxy":
-		output = self._api.low_level.generate(input, self.model, self.params)
-
+	def _create_datablock(self, fragment: Dict[str, str], end_offset: int):
 		story = self._storycontent["data"]["story"]
 		blocks = story["datablocks"]
 		fragments = story["fragments"]
@@ -152,19 +75,17 @@ class NovelAI_StoryProxy:
 
 		story["step"] += 1
 
-		fragment = { "data": output, "origin": "" }
 		frag_index = len(fragments)
 		fragments.append(fragment)
 
-		start = cur_block["endIndex"] + 1
-		end = start + len(output)
+		start = cur_block["endIndex"] + len(cur_block["dataFragment"]["data"])
 
 		block = {
 			"nextBlock": [],
 			"prevBlock": cur_index,
-			"origin": "",
+			"origin": fragment["origin"],
 			"startIndex": start,
-			"endIndex": end,
+			"endIndex": start + end_offset,
 			"dataFragment": fragment,
 			"fragmentIndex": frag_index,
 			"removedFragments": [],
@@ -174,6 +95,18 @@ class NovelAI_StoryProxy:
 		blocks.append(block)
 
 		story["currentBlock"] = new_index
+		self._tree.append(new_index)
+
+	async def generate(self, input: Union[str, List[int]]) -> "NovelAI_StoryProxy":
+		output = await self._api.low_level.generate(input, self.model, self.params)
+		fragment = { "data": output, "origin": "ai" }
+
+		self._create_datablock(fragment, 0)
+
+	async def edit(self, start: int, end: int, replace: str):
+		fragment = { "data": replace, "origin": "edit" }
+
+		self._create_datablock(fragment, end - start)
 
 	async def undo(self):
 		story = self._storycontent["data"]["story"]
@@ -201,40 +134,67 @@ class NovelAI_StoryProxy:
 
 		story["currentBlock"] = next_blocks[index]
 
-	async def flatten(self):
-		pass
+	async def flatten(self) -> NoReturn:
+		story = self._storycontent["data"]["story"]
+
+		blocks = story["datablocks"]
+		new_datablocks = [blocks[i] for i in self._tree]
+		self._tree = [i for i in range(len(new_datablocks))]
+		story["datablocks"] = new_datablocks
 
 	async def delete(self):
 		pass
 
 	async def get_current_tree(self):
-		pass
+		story = self._storycontent["data"]["story"]
+
+		blocks = story["datablocks"]
+		return [blocks[i] for i in self._tree]
 
 class NovelAI_Story:
 	_story_instances: List[NovelAI_StoryProxy]
 
 	_api: NovelAI_API
-	_keystore: Dict[str, bytes]
+	_keystore: Keystore
+	_idstore: Idstore
 	_stories: Dict[str, Dict[str, Any]]
 
-	def __init__(self, api: NovelAI_API, keystore: Dict[str, bytes]):
+	def __init__(self, api: NovelAI_API, keystore: Keystore, idstore: Idstore):
 		self._api = api
 		self._keystore = keystore
 
 	def create(self) -> NovelAI_StoryProxy:
-		# FIXME: make a keystore central storage (create, delete, get) to take care of the garbage collection
-		meta = str(uuid4())
-		while meta in self._keystore:
-			meta = str(uuid4())
-
-		key = random(SecretBox.KEY_SIZE)
-		self._keystore[meta] = key
-
+		meta = self._keystore.create()
 		current_time = get_time()
 		current_time_short = get_short_time()
 
-		story = create_new_stories_data(meta, current_time, current_time_short)
-		storycontent = create_new_storycontent_data(meta, current_time_short)
+		with open("templates/template_empty_story.txt") as f:
+			story = loads(f.read())
+
+		# local overwrites
+		id_story = ""			# FIXME: get id
+		for path, val in (("id", id_story),
+						  ("meta", meta),
+						  ("data.id", meta),
+						  ("data.createdAt", current_time),
+						  ("data.lastUpdatedAt", current_time),
+						  ("lastUpdatedAt", current_time_short)):
+			_set_nested_item(story, val, path)
+
+		with open("templates/template_empty_storycontent.txt") as f:
+			storycontent = loads(f.read())
+
+		# local overwrites
+		id_storycontent = ""	# FIXME: get id
+		id_lore_default = ""	# FIXME: get id
+
+		for path, val in (("id", storycontent_id),
+						  ("meta", meta),
+						  ("lastUpdatedAt", current_time_short),
+						  ("data.contextDefaults.loreDefaults.id", id_lore_default),
+						  ("data.contextDefaults.loreDefaults.lastUpdatedAt", current_time)):
+			_set_nested_item(storycontent, val, path)
+
 
 		proxy = NovelAI_StoryProxy(self, key, story, storycontent)
 		self._story_instances.append(proxy)
@@ -251,7 +211,6 @@ class NovelAI_Story:
 		# FIXME: look for duplicates
 		self._story_instances.append(proxy)
 
-
 	def select(self, id: str) -> NovelAI_StoryProxy:
 		"""
 		Select a story proxy from the previously created/loaded ones
